//------------------------------------------------------------------------------
// File: TransIP2.cpp
//
// Desc: modified transinplace filter
//
// Copyright (c) Microsoft Corporation.  All rights reserved.
// Copyright (c) 2005 N099
//------------------------------------------------------------------------------

#include <streams.h>
#include <measure.h>
#include <dvdmedia.h>
#include "transip2.h"

// =================================================================
// Implements the CPassThruAllocator class
// =================================================================
/* Constructor */
CPassThruAllocator::CPassThruAllocator(TCHAR *pName,
                               LPUNKNOWN pUnk,
                               HRESULT *phr,
                               IMemAllocator * pAllocator,
							   BOOL convert
                               ) : 
	CUnknown(pName, pUnk),
	m_pAllocator(pAllocator),
	m_convert(convert)
{
	DbgLog((LOG_TRACE, 2, TEXT("CPassThruAllocator::CPassThruAllocator")));
}

#ifdef UNICODE
CPassThruAllocator::CPassThruAllocator(CHAR *pName,
                               LPUNKNOWN pUnk,
                               HRESULT *phr,
                               IMemAllocator * pAllocator,
							   BOOL convert) :
	CUnknown(pName, pUnk),
	m_pAllocator(pAllocator),
	m_convert(convert)
{
	DbgLog((LOG_TRACE, 2, TEXT("CPassThruAllocator::CPassThruAllocator")));
}
#endif

/* Destructor */
CPassThruAllocator::~CPassThruAllocator()
{
	DbgLog((LOG_TRACE, 2, TEXT("CPassThruAllocator::~CPassThruAllocator")));
    if (m_pAllocator) {
        m_pAllocator->Release();
    }
}


/* Override this to publicise our interfaces */
STDMETHODIMP
CPassThruAllocator::NonDelegatingQueryInterface(REFIID riid, void **ppv)
{
    /* Do we know about this interface */
    if (riid == IID_IMemAllocator) {
        return GetInterface((IMemAllocator *) this, ppv);
    } else {
        return CUnknown::NonDelegatingQueryInterface(riid, ppv);
    }
}

STDMETHODIMP
CPassThruAllocator::SetProperties(
                ALLOCATOR_PROPERTIES* pRequest,
                ALLOCATOR_PROPERTIES* pActual)
{
    CheckPointer(pRequest, E_POINTER);
    CheckPointer(pActual, E_POINTER);
    ValidateReadWritePtr(pActual, sizeof(ALLOCATOR_PROPERTIES));
 
	ASSERT(m_pAllocator);

	return m_pAllocator->SetProperties(pRequest,pActual);
}

STDMETHODIMP
CPassThruAllocator::GetProperties(
    ALLOCATOR_PROPERTIES * pActual)
{
    CheckPointer(pActual,E_POINTER);
    ValidateReadWritePtr(pActual,sizeof(ALLOCATOR_PROPERTIES));

	ASSERT(m_pAllocator);

	return m_pAllocator->GetProperties(pActual);
}

// convert media type
HRESULT CPassThruAllocator::GetBuffer(IMediaSample **ppBuffer,
                                  REFERENCE_TIME *pStartTime,
                                  REFERENCE_TIME *pEndTime,
                                  DWORD dwFlags
                                  )
{
    UNREFERENCED_PARAMETER(pStartTime);
    UNREFERENCED_PARAMETER(pEndTime);
    UNREFERENCED_PARAMETER(dwFlags);
	ASSERT(m_pAllocator);

	HRESULT hr;
	hr = m_pAllocator->GetBuffer(ppBuffer, pStartTime, pEndTime, dwFlags);
	if (S_OK != hr) {
		return hr;
	}

	// convert media type
	if(m_convert){
		CAutoLock lck(this);
		AM_MEDIA_TYPE *pMT = NULL;

		hr = (*ppBuffer)->GetMediaType(&pMT);
		if (hr == S_OK) {
			if(pMT->formattype == FORMAT_VideoInfo2) {
				hr = ConvertVideoInfo2ToVideoInfo(pMT);
				if (hr == S_OK) {
					hr = (*ppBuffer)->SetMediaType(pMT);
				}
			} else {
				//formattype must be FORMAT_VideoInfo2
                DbgLog((LOG_ERROR, 3, TEXT("CPassThruAllocator::GetBuffer weird formattype")));
			}
			DeleteMediaType(pMT);
		} else if(hr == S_FALSE) {
			//GetMediaType() returns S_FALSE if type not changed
			hr = S_OK;
		}
	}

    return hr;
}


STDMETHODIMP
CPassThruAllocator::ReleaseBuffer(IMediaSample * pSample)
{
    CheckPointer(pSample,E_POINTER);
    ValidateReadPtr(pSample,sizeof(IMediaSample));

	ASSERT(m_pAllocator);

	return m_pAllocator->ReleaseBuffer(pSample);
}


STDMETHODIMP
CPassThruAllocator::Commit()
{
	ASSERT(m_pAllocator);

	return m_pAllocator->Commit();
}


STDMETHODIMP
CPassThruAllocator::Decommit()
{
 	ASSERT(m_pAllocator);

	return m_pAllocator->Decommit();
}


// =================================================================
// Implements the CTransInPlaceFilter2 class
// =================================================================

CTransInPlaceFilter2::CTransInPlaceFilter2
   ( TCHAR     *pName,
     LPUNKNOWN  pUnk,
     REFCLSID   clsid,
     HRESULT   *phr,
     bool       bModifiesData,
	 bool       convert
   )
   : CTransformFilter(pName, pUnk, clsid),
     m_bModifiesData(bModifiesData),
	 m_convert(convert)
{

} // constructor

#ifdef UNICODE
CTransInPlaceFilter2::CTransInPlaceFilter2
   ( CHAR     *pName,
     LPUNKNOWN  pUnk,
     REFCLSID   clsid,
     HRESULT   *phr,
     bool       bModifiesData,
	 bool       convert
   )
   : CTransformFilter(pName, pUnk, clsid),
     m_bModifiesData(bModifiesData),
	 m_convert(convert)
{
#ifdef PERF
    RegisterPerfId();
#endif //  PERF

} // constructor
#endif

CBasePin *
CTransInPlaceFilter2::GetPin(int n)
{
    HRESULT hr = S_OK;

    // Create an input pin if not already done

    if (m_pInput == NULL) {

        m_pInput = new CTransInPlaceInputPin2( NAME("TransInPlace2 input pin")
                                            , this        // Owner filter
                                            , &hr         // Result code
                                            , L"Input"    // Pin name
                                            );

        // Constructor for CTransInPlaceInputPin2 can't fail
        ASSERT(SUCCEEDED(hr));
    }

    // Create an output pin if not already done

    if (m_pInput!=NULL && m_pOutput == NULL) {

        m_pOutput = new CTransInPlaceOutputPin2( NAME("TransInPlace2 output pin")
                                              , this       // Owner filter
                                              , &hr        // Result code
                                              , L"Output"  // Pin name
                                              );

        // a failed return code should delete the object

        ASSERT(SUCCEEDED(hr));
        if (m_pOutput == NULL) {
            delete m_pInput;
            m_pInput = NULL;
        }
    }

    // Return the appropriate pin

    ASSERT (n>=0 && n<=1);
    if (n == 0) {
        return m_pInput;
    } else if (n==1) {
        return m_pOutput;
    } else {
        return NULL;
    }

} // GetPin


//
// DecideBufferSize
//
// Tell the output pin's allocator what size buffers we require.
// *pAlloc will be the allocator our output pin is using.
//

HRESULT CTransInPlaceFilter2::DecideBufferSize
            ( IMemAllocator *pAlloc
            , ALLOCATOR_PROPERTIES *pProperties
            )
{
    ALLOCATOR_PROPERTIES Request, Actual;
    HRESULT hr;

	ASSERT(m_pInput->IsConnected());

    hr = InputPin()->PeekAllocator()->GetProperties(&Request);
    if (FAILED(hr)) {
        // Input connected but with a secretive allocator - enough!
        return hr;
    }

    DbgLog((LOG_MEMORY,1,TEXT("Setting Allocator Requirements")));
    DbgLog((LOG_MEMORY,1,TEXT("Count %d, Size %d"),
           Request.cBuffers, Request.cbBuffer));

    // Pass the allocator requirements to our output side
    // but do a little sanity checking first or we'll just hit
    // asserts in the allocator.

    pProperties->cBuffers = Request.cBuffers;
    pProperties->cbBuffer = Request.cbBuffer;
    pProperties->cbAlign = Request.cbAlign;
    if (pProperties->cBuffers<=0) {pProperties->cBuffers = 1; }
    if (pProperties->cbBuffer<=0) {pProperties->cbBuffer = 1; }
    hr = pAlloc->SetProperties(pProperties, &Actual);

    if (FAILED(hr)) {
        return hr;
    }

    DbgLog((LOG_MEMORY,1,TEXT("Obtained Allocator Requirements")));
    DbgLog((LOG_MEMORY,1,TEXT("Count %d, Size %d, Alignment %d"),
           Actual.cBuffers, Actual.cbBuffer, Actual.cbAlign));

    // Make sure we got the right alignment and at least the minimum required

    if (  (Request.cBuffers > Actual.cBuffers)
       || (Request.cbBuffer > Actual.cbBuffer)
       || (Request.cbAlign  > Actual.cbAlign)
       ) {
        return E_FAIL;
    }
    return NOERROR;

} // DecideBufferSize

//
// Copy
//
// return a pointer to an identical copy of pSample
IMediaSample * CTransInPlaceFilter2::Copy(IMediaSample *pSource)
{
    IMediaSample * pDest;

    HRESULT hr;
    REFERENCE_TIME tStart, tStop;
    const BOOL bTime = S_OK == pSource->GetTime( &tStart, &tStop);

    // this may block for an indeterminate amount of time
    hr = OutputPin()->PeekAllocator()->GetBuffer(
              &pDest
              , bTime ? &tStart : NULL
              , bTime ? &tStop : NULL
              , m_bSampleSkipped ? AM_GBF_PREVFRAMESKIPPED : 0
              );

    if (FAILED(hr)) {
        return NULL;
    }

    ASSERT(pDest);
    IMediaSample2 *pSample2;
    if (SUCCEEDED(pDest->QueryInterface(IID_IMediaSample2, (void **)&pSample2))) {
        HRESULT hr = pSample2->SetProperties(
            FIELD_OFFSET(AM_SAMPLE2_PROPERTIES, pbBuffer),
            (PBYTE)m_pInput->SampleProps());
        pSample2->Release();
        if (FAILED(hr)) {
            pDest->Release();
            return NULL;
        }
    } else {
        if (bTime) {
            pDest->SetTime(&tStart, &tStop);
        }

        if (S_OK == pSource->IsSyncPoint()) {
            pDest->SetSyncPoint(TRUE);
        }
        if (S_OK == pSource->IsDiscontinuity() || m_bSampleSkipped) {
            pDest->SetDiscontinuity(TRUE);
        }
        if (S_OK == pSource->IsPreroll()) {
            pDest->SetPreroll(TRUE);
        }

        // Copy the media type
        AM_MEDIA_TYPE *pMediaType;
        if (S_OK == pSource->GetMediaType(&pMediaType)) {
            pDest->SetMediaType(pMediaType);
            DeleteMediaType( pMediaType );
        }

    }

    m_bSampleSkipped = FALSE;

    // Copy the sample media times
    REFERENCE_TIME TimeStart, TimeEnd;
    if (pSource->GetMediaTime(&TimeStart,&TimeEnd) == NOERROR) {
        pDest->SetMediaTime(&TimeStart,&TimeEnd);
    }

    // Copy the actual data length and the actual data.
    {
        const long lDataLength = pSource->GetActualDataLength();
        pDest->SetActualDataLength(lDataLength);

        // Copy the sample data
        {
            BYTE *pSourceBuffer, *pDestBuffer;
            long lSourceSize  = pSource->GetSize();
            long lDestSize = pDest->GetSize();

            ASSERT(lDestSize >= lSourceSize && lDestSize >= lDataLength);

            pSource->GetPointer(&pSourceBuffer);
            pDest->GetPointer(&pDestBuffer);
            ASSERT(lDestSize == 0 || pSourceBuffer != NULL && pDestBuffer != NULL);

            CopyMemory( (PVOID) pDestBuffer, (PVOID) pSourceBuffer, lDataLength );
        }
    }

    return pDest;

} // Copy


// override this to customize the transform process

HRESULT
CTransInPlaceFilter2::Receive(IMediaSample *pSample)
{
    /*  Check for other streams and pass them on */
    AM_SAMPLE2_PROPERTIES * const pProps = m_pInput->SampleProps();
    if (pProps->dwStreamId != AM_STREAM_MEDIA) {
        return m_pOutput->Deliver(pSample);
    }
    HRESULT hr;

    // Start timing the TransInPlace (if PERF is defined)
    MSR_START(m_idTransInPlace);

    if (UsingDifferentAllocators()) {

        // We have to copy the data.

        pSample = Copy(pSample);

        if (pSample==NULL) {
            MSR_STOP(m_idTransInPlace);
            return E_UNEXPECTED;
        }
    }

    // have the derived class transform the data
    hr = Transform(pSample);

    // Stop the clock and log it (if PERF is defined)
    MSR_STOP(m_idTransInPlace);

    if (FAILED(hr)) {
        DbgLog((LOG_TRACE, 1, TEXT("Error from TransInPlace")));
        if (UsingDifferentAllocators()) {
            pSample->Release();
        }
        return hr;
    }

    // the Transform() function can return S_FALSE to indicate that the
    // sample should not be delivered; we only deliver the sample if it's
    // really S_OK (same as NOERROR, of course.)
    if (hr == NOERROR) {
        hr = m_pOutput->Deliver(pSample);
    } else {
        //  But it would be an error to return this private workaround
        //  to the caller ...
        if (S_FALSE == hr) {
            // S_FALSE returned from Transform is a PRIVATE agreement
            // We should return NOERROR from Receive() in this cause because
            // returning S_FALSE from Receive() means that this is the end
            // of the stream and no more data should be sent.
            m_bSampleSkipped = TRUE;
            if (!m_bQualityChanged) {
                NotifyEvent(EC_QUALITY_CHANGE,0,0);
                m_bQualityChanged = TRUE;
            }
            hr = NOERROR;
        }
    }

    // release the output buffer. If the connected pin still needs it,
    // it will have addrefed it itself.
    if (UsingDifferentAllocators()) {
        pSample->Release();
    }

    return hr;

} // Receive



// =================================================================
// Implements the CTransInPlaceInputPin2 class
// =================================================================


// constructor

CTransInPlaceInputPin2::CTransInPlaceInputPin2
    ( TCHAR               *pObjectName
    , CTransInPlaceFilter2 *pFilter
    , HRESULT             *phr
    , LPCWSTR              pName
    )
    : CTransformInputPin(pObjectName,
                         pFilter,
                         phr,
                         pName)
    , m_bReadOnly(FALSE)
    , m_UsingPassThruAllocator(FALSE)
    , m_pTIPFilter(pFilter)
{
    DbgLog((LOG_TRACE, 2
           , TEXT("CTransInPlaceInputPin2::CTransInPlaceInputPin2")));

} // constructor


// =================================================================
// Implements IMemInputPin interface
// =================================================================


// If the downstream filter has one then offer that (even if our own output
// pin is not using it yet.  If the upstream filter chooses it then we will
// tell our output pin to ReceiveAllocator).
// Else if our output pin is using an allocator then offer that.
//     ( This could mean offering the upstream filter his own allocator,
//       it could mean offerring our own
//     ) or it could mean offering the one from downstream
// Else fail to offer any allocator at all.

STDMETHODIMP CTransInPlaceInputPin2::GetAllocator(IMemAllocator ** ppAllocator)
{
    CheckPointer(ppAllocator,E_POINTER);
    ValidateReadWritePtr(ppAllocator,sizeof(IMemAllocator *));
    CAutoLock cObjectLock(m_pLock);

    HRESULT hr;

    if ( m_pTIPFilter->m_pOutput->IsConnected() ) {
        //  Store the allocator we got
        hr = m_pTIPFilter->OutputPin()->ConnectedIMemInputPin()
                                        ->GetAllocator( ppAllocator );
        if (SUCCEEDED(hr)) {
			// Get custom allocator
            m_pTIPFilter->OutputPin()->SetAllocator( *ppAllocator );
			CPassThruAllocator *pAlloc = new CPassThruAllocator(
				NAME("CPassThruAllocator"), NULL, &hr, *ppAllocator, m_pTIPFilter->m_convert);
			if (!pAlloc) {
				return E_OUTOFMEMORY;
			}
			if (FAILED(hr)) {
				delete pAlloc;
				return hr;
			}
			(*ppAllocator)->AddRef();
			hr = pAlloc->QueryInterface(IID_IMemAllocator, (void**)&m_pAllocator);
			if (FAILED(hr)) {
				delete pAlloc;
				return hr;
			}
			ASSERT(m_pAllocator != NULL);
			*ppAllocator = m_pAllocator;
			m_pAllocator->AddRef();
			m_UsingPassThruAllocator = TRUE;
			return NOERROR;
        }
    }
    else {
        //  Help upstream filter (eg TIP filter which is having to do a copy)
        //  by providing a temp allocator here - we'll never use
        //  this allocator because when our output is connected we'll
        //  reconnect this pin
        hr = CTransformInputPin::GetAllocator( ppAllocator );
    }
    return hr;

} // GetAllocator



/* Get told which allocator the upstream output pin is actually going to use */


STDMETHODIMP
CTransInPlaceInputPin2::NotifyAllocator(
    IMemAllocator * pAllocator,
    BOOL bReadOnly)
{
    HRESULT hr = S_OK;
    CheckPointer(pAllocator,E_POINTER);
    ValidateReadPtr(pAllocator,sizeof(IMemAllocator));

    CAutoLock cObjectLock(m_pLock);

    m_bReadOnly = bReadOnly;
    //  If we modify data then don't accept the allocator if it's
    //  the same as the output pin's allocator

    //  If our output is not connected just accept the allocator
    //  We're never going to use this allocator because when our
    //  output pin is connected we'll reconnect this pin
    if (!m_pTIPFilter->OutputPin()->IsConnected()) {
        return CTransformInputPin::NotifyAllocator(pAllocator, bReadOnly);
    }

    //  If the allocator is read-only and we're modifying data
    //  and the allocator is the same as the output pin's
    //  then reject
    if (bReadOnly && m_pTIPFilter->m_bModifiesData) {
        IMemAllocator *pOutputAllocator =
            m_pTIPFilter->OutputPin()->PeekAllocator();

        //  Make sure we have an output allocator
        if (pOutputAllocator == NULL) {
            hr = m_pTIPFilter->OutputPin()->ConnectedIMemInputPin()->
                                      GetAllocator(&pOutputAllocator);
            if(FAILED(hr)) {
                hr = CreateMemoryAllocator(&pOutputAllocator);
            }
            if (SUCCEEDED(hr)) {
                m_pTIPFilter->OutputPin()->SetAllocator(pOutputAllocator);
                pOutputAllocator->Release();
            }
        }
        if (pAllocator == pOutputAllocator) {
            hr = E_FAIL;
        } else if(SUCCEEDED(hr)) {
            //  Must copy so set the allocator properties on the output
            ALLOCATOR_PROPERTIES Props, Actual;
            hr = pAllocator->GetProperties(&Props);
            if (SUCCEEDED(hr)) {
                hr = pOutputAllocator->SetProperties(&Props, &Actual);
            }
            if (SUCCEEDED(hr)) {
                if (  (Props.cBuffers > Actual.cBuffers)
                   || (Props.cbBuffer > Actual.cbBuffer)
                   || (Props.cbAlign  > Actual.cbAlign)
                   ) {
                    hr =  E_FAIL;
                }
            }

            //  Set the allocator on the output pin
            if (SUCCEEDED(hr)) {
                hr = m_pTIPFilter->OutputPin()->ConnectedIMemInputPin()
                                       ->NotifyAllocator( pOutputAllocator, FALSE );
            }
        }
    }


    if (SUCCEEDED(hr)) {
		IMemAllocator *pOldAllocator = m_pAllocator;
		if(pAllocator == m_pAllocator) { //using custom_allocater
			DbgLog((LOG_TRACE, 2, TEXT("CTransInPlaceInputPin2::NotifyAllocator use my allocater")));

			m_UsingPassThruAllocator = TRUE;
		} else {
			DbgLog((LOG_TRACE, 2, TEXT("CTransInPlaceInputPin2::NotifyAllocator not use my alloc")));

			m_UsingPassThruAllocator = FALSE;
			pAllocator->AddRef();
			m_pAllocator = pAllocator;

			if(pOldAllocator != NULL){
				pOldAllocator->Release();
			}
		}
    }

    return hr;

} // NotifyAllocator


// CheckMediaType
HRESULT CTransInPlaceInputPin2::CheckMediaType(const CMediaType *pmt )
{
    HRESULT hr = m_pTIPFilter->CheckInputType(pmt);
    if (hr!=S_OK) return hr;

    if( m_pTIPFilter->m_pOutput->IsConnected() )
        return m_pTIPFilter->m_pOutput->GetConnected()->QueryAccept( pmt );
    else
        return S_OK;

} // CheckMediaType


// If upstream asks us what our requirements are, we will try to ask downstream
// if that doesn't work, we'll just take the defaults.
STDMETHODIMP
CTransInPlaceInputPin2::GetAllocatorRequirements(ALLOCATOR_PROPERTIES *pProps)
{

    if( m_pTIPFilter->m_pOutput->IsConnected() )
        return m_pTIPFilter->OutputPin()
               ->ConnectedIMemInputPin()->GetAllocatorRequirements( pProps );
    else
        return E_NOTIMPL;

} // GetAllocatorRequirements


// CTransInPlaceInputPin2::CompleteConnect() calls CBaseInputPin::CompleteConnect()
// and then calls CTransInPlaceFilter2::CompleteConnect().  It does this because 
// CTransInPlaceFilter2::CompleteConnect() can reconnect a pin and we do not
// want to reconnect a pin if CBaseInputPin::CompleteConnect() fails.
HRESULT
CTransInPlaceInputPin2::CompleteConnect(IPin *pReceivePin)
{
	HRESULT hr;

	hr = CBaseInputPin::CompleteConnect(pReceivePin);
    if (FAILED(hr)) {
        return hr;
    }

    return m_pTransformFilter->CompleteConnect(PINDIR_INPUT,pReceivePin);
} // CompleteConnect

//
IMemAllocator * CTransInPlaceInputPin2::PeekAllocator(void)
{
	if(m_pTIPFilter->m_pOutput->IsConnected() && m_UsingPassThruAllocator){
		return static_cast<CPassThruAllocator*>(m_pAllocator)->PeekAllocator();
	}
	m_UsingPassThruAllocator = FALSE;

	return m_pAllocator; 
}

//
HRESULT CTransInPlaceInputPin2::SetConvert(BOOL cnv)
{
	if(m_pTIPFilter->m_pOutput->IsConnected() && m_UsingPassThruAllocator){
		static_cast<CPassThruAllocator*>(m_pAllocator)->SetConvert(cnv);
		return S_OK;
	}
	m_UsingPassThruAllocator = FALSE;

	return S_FALSE;
}

// =================================================================
// Implements the CTransInPlaceOutputPin2 class
// =================================================================


// constructor

CTransInPlaceOutputPin2::CTransInPlaceOutputPin2(
    TCHAR *pObjectName,
    CTransInPlaceFilter2 *pFilter,
    HRESULT * phr,
    LPCWSTR pPinName)
    : CTransformOutputPin( pObjectName
                         , pFilter
                         , phr
                         , pPinName),
      m_pTIPFilter(pFilter)
{
    DbgLog(( LOG_TRACE, 2
           , TEXT("CTransInPlaceOutputPin2::CTransInPlaceOutputPin2")));

} // constructor


// CheckMediaType
HRESULT CTransInPlaceOutputPin2::CheckMediaType(const CMediaType *pmt )
{
    // Don't accept any output pin type changes if we're copying
    // between allocators - it's too late to change the input
    // allocator size.
	
    if (m_pTIPFilter->UsingDifferentAllocators() && !m_pFilter->IsStopped()) {
        if (*pmt == m_mt) {
            return S_OK;
        } else {
            return VFW_E_TYPE_NOT_ACCEPTED;
        }
    }
	

    // Assumes the type does not change.  That's why we're calling
    // CheckINPUTType here on the OUTPUT pin.
    HRESULT hr = m_pTIPFilter->CheckInputType(pmt);
    if (hr!=S_OK) return hr;
		
	if( m_pTIPFilter->m_pInput->IsConnected() ){
		hr = m_pTIPFilter->m_pInput->GetConnected()->QueryAccept( pmt );
		if (S_OK == hr) {
			// VideoInfo2(or other) is accepted
			DbgLog((LOG_TRACE, 3, TEXT("CTIPlOutputPin2::CheckMediaType vif2 accepted")));
			m_pTIPFilter->m_convert = false;
			static_cast<CTransInPlaceInputPin2*>(m_pTIPFilter->m_pInput)->SetConvert(false);
			//m_pTIPFilter->m_pInput->SetMediaType(pmt);
			//m_pTIPFilter->m_pOutput->SetMediaType(pmt);
			return hr;
		}
		// Accept only VideoInfo
		if(*pmt->FormatType() != FORMAT_VideoInfo2){
			DbgLog((LOG_TRACE, 3, TEXT("CTIPlOutputPin2::CheckMediaType not vif2!")));
			return VFW_E_TYPE_NOT_ACCEPTED;
		}
		
		// convert media type 
		if ( m_pTIPFilter->m_convert ) {
			AM_MEDIA_TYPE amt;

			if(S_OK == CopyMediaType(&amt, pmt)){
				// convert to VideoInfo
				ConvertVideoInfo2ToVideoInfo(&amt);
				hr = m_pTIPFilter->m_pInput->GetConnected()->QueryAccept( &amt );
				if (S_OK == hr) {
					CMediaType cmt(amt);
					// VideoInfo is accepted
					DbgLog((LOG_TRACE, 3, TEXT("CTIPlOutputPin2::CheckMediaType vif accepted")));
					static_cast<CTransInPlaceInputPin2*>(m_pTIPFilter->m_pInput)->SetConvert(true);
					m_pTIPFilter->m_pInput->SetMediaType(&cmt);
					//m_pTIPFilter->m_pOutput->SetMediaType(&cmt);
				}
				FreeMediaType( amt );
			}
		}
		
		return hr;
	}
	else
        return S_OK;

} // CheckMediaType


/* Save the allocator pointer in the output pin
*/
void
CTransInPlaceOutputPin2::SetAllocator(IMemAllocator * pAllocator)
{
    pAllocator->AddRef();
    if (m_pAllocator) {
        m_pAllocator->Release();
    }
    m_pAllocator = pAllocator;
} // SetAllocator


// CTransInPlaceOutputPin2::CompleteConnect() calls CBaseOutputPin::CompleteConnect()
// and then calls CTransInPlaceFilter2::CompleteConnect().  It does this because 
// CTransInPlaceFilter2::CompleteConnect() can reconnect a pin and we do not want to 
// reconnect a pin if CBaseOutputPin::CompleteConnect() fails.  
// CBaseOutputPin::CompleteConnect() often fails when our output pin is being connected 
// to the Video Mixing Renderer.
HRESULT
CTransInPlaceOutputPin2::CompleteConnect(IPin *pReceivePin)
{
    HRESULT hr = CBaseOutputPin::CompleteConnect(pReceivePin);
    if (FAILED(hr)) {
        return hr;
    }

    return m_pTransformFilter->CompleteConnect(PINDIR_OUTPUT,pReceivePin);
} // CompleteConnect


/*  inverse function for ConvertVideoInfoToVideoInfo2
*/
STDAPI ConvertVideoInfo2ToVideoInfo(AM_MEDIA_TYPE *pmt)
{
    ASSERT(pmt->formattype == FORMAT_VideoInfo2);
    PVOID pvNew = CoTaskMemAlloc(pmt->cbFormat + sizeof(VIDEOINFOHEADER) -
                                 sizeof(VIDEOINFOHEADER2));
    if (pvNew == NULL) {
        return E_OUTOFMEMORY;
    }
    CopyMemory(pvNew, pmt->pbFormat, FIELD_OFFSET(VIDEOINFOHEADER, bmiHeader));
    CopyMemory((PBYTE)pvNew + FIELD_OFFSET(VIDEOINFOHEADER, bmiHeader),
               pmt->pbFormat + FIELD_OFFSET(VIDEOINFOHEADER2, bmiHeader),
               pmt->cbFormat - FIELD_OFFSET(VIDEOINFOHEADER2, bmiHeader));
    pmt->formattype = FORMAT_VideoInfo;
    CoTaskMemFree(pmt->pbFormat);
    pmt->pbFormat = (PBYTE)pvNew;
    pmt->cbFormat -= sizeof(VIDEOINFOHEADER2) - sizeof(VIDEOINFOHEADER);
    return S_OK;
}
